﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Net;
using System.IO;
using System.Xml;

namespace DaHanSmsDemo_CSharp
{
    enum EBodyType : uint
    {
        EType_JSON = 0,
        EType_XML
    };
    class SmsSDK
    {
        private string apiUrl = null;
        private string accountSid = null;
        private string token = null;
        private string appId = null;

        private EBodyType bodyType = EBodyType.EType_JSON;

    

        /// <summary>
        /// 设置服务器地址和端口信息
        /// </summary>
        /// <param name="apiUrl">ip和端口</param>
        public void setApiUrl(string apiUrl)
        {
            this.apiUrl = apiUrl;
        }

        /// <summary>
        /// 设置主帐号信息
        /// </summary>
        /// <param name="accountSid">主帐号</param>
        public void setAccountSid(string accountSid)
        {
            this.accountSid = accountSid;
        }

        /// <summary>
        /// 设置主帐号信息
        /// </summary>
        /// <param name="token">主帐号令牌</param>
        public void setAuthToken(string accountToken)
        {
            this.token = accountToken;
        }

        /// <summary>
        /// 设置应用ID
        /// </summary>
        /// <param name="appId">应用ID</param>
        public void setAppId(string appId)
        {
            this.appId = appId;
        }

        /// <summary>
        /// 模板短信
        /// </summary>
        /// <param name="to">短信接收端手机号码集合，用英文逗号分开，每批发送的手机号数量不得超过100个</param>
        /// <param name="templateId">模板Id</param>
        /// <param name="datas">可选字段 内容数据，用于替换模板中{序号}</param>
        /// <exception cref="ArgumentNullException">参数不能为空</exception>
        /// <exception cref="Exception"></exception>
        /// <returns></returns>
        public Dictionary<string, object> sendSMS(string to, string templateId, string[] datas)
        {
            Dictionary<string, object> initError = check();
            if(initError != null)
            {
                return initError;
            }
            initError = checkAccount();
            if (initError != null)
            {
                return initError;
            }
            initError = checkAppId();
            if (initError != null)
            {
                return initError;
            }
            initError = checkTo(to);
            if (initError != null)
            {
                return initError;
            }
            initError = checkTemplateId(templateId);
            if (initError != null)
            {
                return initError;
            }

            try
            {
                string date = DateTime.Now.ToString("yyyyMMddhhmmss");

                //构建URL内容
                string sig = MD5Encrypt(accountSid + token + date);
                string sendUrl = string.Format("{0}API/sendMessage?sig={1}", apiUrl, sig);
                Uri url = new Uri(sendUrl);

                Console.WriteLine(url);

                //创建网络请求
                HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
                setCertificateValidationCallBack();

                //构建Head
                request.Method = "POST";

                Encoding myEncoding = Encoding.GetEncoding("utf-8");
                byte[] myByte = myEncoding.GetBytes(accountSid + ":" + date);
                string authorization = Convert.ToBase64String(myByte);
                request.Headers.Add("authorization", authorization);

                //构建Body
                StringBuilder bodyData = new StringBuilder();

                if (bodyType == EBodyType.EType_JSON)
                {
                    request.Accept = "application/json";
                    request.ContentType = "application/json;charset=utf-8";

                    bodyData.Append("{");
                    bodyData.Append("\"to\":\"").Append(to).Append("\"");
                    bodyData.Append(",\"appId\":\"").Append(appId).Append("\"");
                    bodyData.Append(",\"templateId\":\"").Append(templateId).Append("\"");
                    if (datas != null && datas.Length > 0)
                    {
                        bodyData.Append(",\"datas\":[");
                        int index = 0;
                        foreach (string item in datas)
                        {
                            if (index == 0)
                            {
                                bodyData.Append("\"" + item + "\"");
                            }
                            else
                            {
                                bodyData.Append(",\"" + item + "\"");
                            }
                            index++;
                        }
                        bodyData.Append("]");
                    }
                    bodyData.Append("}");
                }
                else
                {
                    request.Accept = "application/xml";
                    request.ContentType = "application/xml;charset=utf-8";

                    bodyData.Append("<?xml version='1.0' encoding='utf-8'?><TemplateSMS>");
                    bodyData.Append("<to>").Append(to).Append("</to>");
                    bodyData.Append("<appId>").Append(appId).Append("</appId>");
                    bodyData.Append("<templateId>").Append(templateId).Append("</templateId>");
                    if (datas != null && datas.Length > 0)
                    {
                        bodyData.Append("<datas>");
                        foreach (string item in datas)
                        {
                            bodyData.Append("<datas>").Append(item).Append("</datas>");
                        }
                        bodyData.Append("</datas>");
                    }
                    bodyData.Append("</TemplateSMS>");
                }

                byte[] byteData = UTF8Encoding.UTF8.GetBytes(bodyData.ToString());

                Console.WriteLine("SendTemplateSMS requestBody = " + bodyData.ToString());

                //开始请求
                using(Stream postStream = request.GetRequestStream())
                {
                    postStream.Write(byteData, 0, byteData.Length);
                }

                //获取请求
                using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
                {
                    // Get the response stream  
                    StreamReader reader = new StreamReader(response.GetResponseStream());
                    string responseStr = reader.ReadToEnd();

                    Console.WriteLine("SendTemplateSMS responseBody = " + responseStr);
                    if (responseStr != null && responseStr.Length > 0)
                    {
                        Dictionary<string, object> responseResult = new Dictionary<string, object> { { "statusCode", "000000" }, { "statusMsg", "成功" }, { "templateSMS", null } };

                        if (bodyType == EBodyType.EType_XML)
                        {
                            XmlDocument resultXml = new XmlDocument();
                            resultXml.LoadXml(responseStr);
                            XmlNodeList nodeList = resultXml.SelectSingleNode("Response").ChildNodes;
                            foreach (XmlNode item in nodeList)
                            {
                                if (item.Name == "statusCode")
                                {
                                    responseResult["statusCode"] = item.InnerText;
                                }
                                else if (item.Name == "statusMsg")
                                {
                                    responseResult["statusMsg"] = item.InnerText;
                                }
                                else if (item.Name == "TemplateSMS")
                                {
                                    Dictionary<string, object> retData = new Dictionary<string, object>();
                                    foreach (XmlNode subItem in item.ChildNodes)
                                    {
                                        retData.Add(subItem.Name, subItem.InnerText);
                                    }
                                    responseResult["templateSMS"] = new Dictionary<string, object> { { item.Name, retData } };
                                }
                            }
                        }
                        else
                        {
                            responseResult.Clear();
                            responseResult["resposeBody"] = responseStr;
                        }

                        return responseResult;
                    }
                    return new Dictionary<string, object> { { "statusCode", "000010" }, { "statusMsg", "【服务器】服务器请求出错，请确认参数是否正确，如无法解决，请联系工作人员" }, { "templateSMS", null } };
                }


            }
            catch (Exception e)
            {
                throw e;
            }
        }


        /// <summary>
        /// 模板短信
        /// </summary>
        /// <param name="to">短信接收端手机号码集合，用英文逗号分开，每批发送的手机号数量不得超过100个</param>
        /// <param name="templateId">模板Id</param>
        /// <param name="datas">可选字段 内容数据，用于替换模板中{序号}</param>
        /// <exception cref="ArgumentNullException">参数不能为空</exception>
        /// <exception cref="Exception"></exception>
        /// <returns></returns>
        public Dictionary<string, object> sendINTSMS(string to, string templateId, string[] datas)
        {
            Dictionary<string, object> initError = check();
            if (initError != null)
            {
                return initError;
            }
            initError = checkAccount();
            if (initError != null)
            {
                return initError;
            }
            initError = checkAppId();
            if (initError != null)
            {
                return initError;
            }
            initError = checkTo(to);
            if (initError != null)
            {
                return initError;
            }
            initError = checkTemplateId(templateId);
            if (initError != null)
            {
                return initError;
            }

            try
            {
                string date = DateTime.Now.ToString("yyyyMMddhhmmss");

                //构建URL内容
                string sig = MD5Encrypt(accountSid + token + date);
                string sendUrl = string.Format("{0}api/intsms?sig={1}", apiUrl, sig);
                Uri url = new Uri(sendUrl);

                Console.WriteLine(url);

                //创建网络请求
                HttpWebRequest request = WebRequest.Create(url) as HttpWebRequest;
                setCertificateValidationCallBack();

                //构建Head
                request.Method = "POST";

                Encoding myEncoding = Encoding.GetEncoding("utf-8");
                byte[] myByte = myEncoding.GetBytes(accountSid + ":" + date);
                string authorization = Convert.ToBase64String(myByte);
                request.Headers.Add("authorization", authorization);

                //构建Body
                StringBuilder bodyData = new StringBuilder();

                if (bodyType == EBodyType.EType_JSON)
                {
                    request.Accept = "application/json";
                    request.ContentType = "application/json;charset=utf-8";

                    bodyData.Append("{");
                    bodyData.Append("\"to\":\"").Append(to).Append("\"");
                    bodyData.Append(",\"appId\":\"").Append(appId).Append("\"");
                    bodyData.Append(",\"templateId\":\"").Append(templateId).Append("\"");
                    if (datas != null && datas.Length > 0)
                    {
                        bodyData.Append(",\"datas\":[");
                        int index = 0;
                        foreach (string item in datas)
                        {
                            if (index == 0)
                            {
                                bodyData.Append("\"" + item + "\"");
                            }
                            else
                            {
                                bodyData.Append(",\"" + item + "\"");
                            }
                            index++;
                        }
                        bodyData.Append("]");
                    }
                    bodyData.Append("}");
                }
                else
                {
                    request.Accept = "application/xml";
                    request.ContentType = "application/xml;charset=utf-8";

                    bodyData.Append("<?xml version='1.0' encoding='utf-8'?><TemplateSMS>");
                    bodyData.Append("<to>").Append(to).Append("</to>");
                    bodyData.Append("<appId>").Append(appId).Append("</appId>");
                    bodyData.Append("<templateId>").Append(templateId).Append("</templateId>");
                    if (datas != null && datas.Length > 0)
                    {
                        bodyData.Append("<datas>");
                        foreach (string item in datas)
                        {
                            bodyData.Append("<datas>").Append(item).Append("</datas>");
                        }
                        bodyData.Append("</datas>");
                    }
                    bodyData.Append("</TemplateSMS>");
                }

                byte[] byteData = UTF8Encoding.UTF8.GetBytes(bodyData.ToString());

                Console.WriteLine("SendTemplateSMS requestBody = " + bodyData.ToString());

                //开始请求
                using (Stream postStream = request.GetRequestStream())
                {
                    postStream.Write(byteData, 0, byteData.Length);
                }

                //获取请求
                using (HttpWebResponse response = request.GetResponse() as HttpWebResponse)
                {
                    // Get the response stream  
                    StreamReader reader = new StreamReader(response.GetResponseStream());
                    string responseStr = reader.ReadToEnd();

                    Console.WriteLine("SendTemplateSMS responseBody = " + responseStr);
                    if (responseStr != null && responseStr.Length > 0)
                    {
                        Dictionary<string, object> responseResult = new Dictionary<string, object> { { "statusCode", "000000" }, { "statusMsg", "成功" }, { "templateSMS", null } };

                        if (bodyType == EBodyType.EType_XML)
                        {
                            XmlDocument resultXml = new XmlDocument();
                            resultXml.LoadXml(responseStr);
                            XmlNodeList nodeList = resultXml.SelectSingleNode("Response").ChildNodes;
                            foreach (XmlNode item in nodeList)
                            {
                                if (item.Name == "statusCode")
                                {
                                    responseResult["statusCode"] = item.InnerText;
                                }
                                else if (item.Name == "statusMsg")
                                {
                                    responseResult["statusMsg"] = item.InnerText;
                                }
                                else if (item.Name == "TemplateSMS")
                                {
                                    Dictionary<string, object> retData = new Dictionary<string, object>();
                                    foreach (XmlNode subItem in item.ChildNodes)
                                    {
                                        retData.Add(subItem.Name, subItem.InnerText);
                                    }
                                    responseResult["templateSMS"] = new Dictionary<string, object> { { item.Name, retData } };
                                }
                            }
                        }
                        else
                        {
                            responseResult.Clear();
                            responseResult["resposeBody"] = responseStr;
                        }

                        return responseResult;
                    }
                    return new Dictionary<string, object> { { "statusCode", "000010" }, { "statusMsg", "【服务器】服务器请求出错，请确认参数是否正确，如无法解决，请联系工作人员" }, { "templateSMS", null } };
                }


            }
            catch (Exception e)
            {
                throw e;
            }
        }



        /// <summary>
        /// 检查服务器地址信息
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, object> check()
        {
            String statusCode = "000000";
            string statusMsg = null;

            if(apiUrl == null)
            {
                statusCode = "000019";
                statusMsg = "【短信】url空";
            }

            if( statusCode != "000000")
            {
                return new Dictionary<string, object>
                {
                    {"statusCode", statusCode + "" },
                    {"statusMsg", statusMsg },
                    {"templateSMS", null }
                };
            }

            return null;
        }
        /// <summary>
        /// 检查主帐号信息
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, object> checkAccount()
        {
            string statusCode = "000000";
            string statusMsg = null;

            if (accountSid == null)
            {
                statusCode = "000017";
                statusMsg = "【账号】帐号空";
            }
            else if (token == null)
            {
                statusCode = "000018";
                statusMsg = "【账号】帐号令牌空 ";
            }

            if (statusCode != "000000")
            {
                return new Dictionary<string, object>
                {
                    {"statusCode", statusCode + "" },
                    {"statusMsg", statusMsg },
                    {"templateSMS", null }
                };
            }

            return null;
        }
        /// <summary>
        /// 检查应用ID
        /// </summary>
        /// <returns></returns>
        private Dictionary<string, object> checkAppId()
        {
            if (appId == null)
            {
                return new Dictionary<string, object> { { "statusCode", "000016" }, { "statusMsg", "【应用】应用ID为空" }, { "templateSMS", null } };
            }

            return null;
        }

        private Dictionary<string, object> checkTo(string to)
        {
            if (to == null)
            {
                return new Dictionary<string, object> { { "statusCode", "000006" }, { "statusMsg", "【短信】接收短信的手机号码为空" }, { "templateSMS", null } };
            }

            return null;
        }

        private Dictionary<string, object> checkTemplateId(string templateId)
        {
            if (templateId == null)
            {
                return new Dictionary<string, object> { { "statusCode", "000008" }, { "statusMsg", "【短信】短信模板为空或未设置" }, { "templateSMS", null } };
            }

            return null;
        }

        /// <summary>
        /// MD5加密
        /// </summary>
        /// <param name="source">原内容</param>
        /// <returns>加密后内容</returns>
        public static string MD5Encrypt(string source)
        {
            // Create a new instance of the MD5CryptoServiceProvider object.
            System.Security.Cryptography.MD5 md5Hasher = System.Security.Cryptography.MD5.Create();

            // Convert the input string to a byte array and compute the hash.
            byte[] templateSMS = md5Hasher.ComputeHash(Encoding.Default.GetBytes(source));

            // Create a new Stringbuilder to collect the bytes and create a string.
            StringBuilder sBuilder = new StringBuilder();

            // Loop through each byte of the hashed data and format each one as a hexadecimal string.
            for (int i = 0; i < templateSMS.Length; i++)
            {
                sBuilder.Append(templateSMS[i].ToString("X2"));
            }

            // Return the hexadecimal string.
            return sBuilder.ToString();
        }

        /// <summary>
        /// 设置服务器证书验证回调
        /// </summary>
        public void setCertificateValidationCallBack()
        {
            System.Net.ServicePointManager.ServerCertificateValidationCallback = CertificateValidationResult;
        }

        /// <summary>
        ///  证书验证回调函数  
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="cer"></param>
        /// <param name="chain"></param>
        /// <param name="error"></param>
        /// <returns></returns>
        public bool CertificateValidationResult(object obj, System.Security.Cryptography.X509Certificates.X509Certificate cer, System.Security.Cryptography.X509Certificates.X509Chain chain, System.Net.Security.SslPolicyErrors error)
        {
            return true;
        }

      
    }
}
